import express from 'express';

import MaterialValidate from './validate/material';
import MaterialModel from '../models/material'
import IdValidate from './validate/id';


const router = express.Router();

router.get('/materials', (req, res, next) => {
  if (!req.auth(res, ['read', 'write', 'maintain', 'dev', 'admin'], 'all')) return;

  MaterialModel.find({}).lean().exec((err, data) => {
    if(err) next(err);
    res.json(data.map(e => MaterialValidate.output(e)).filter(e => e !== null));  // validate all and filter null values from validation errors
  });
});

router.get('/material/' + IdValidate.parameter(), (req, res, next) => {
  if (!req.auth(res, ['read', 'write', 'maintain', 'dev', 'admin'], 'all')) return;

  MaterialModel.findById(req.params.id).lean().exec((err, data) => {
    if(err) next(err);
    console.log(data);
    if (data) {
      res.json(MaterialValidate.output(data));
    }
    else {
      res.status(404).json({status: 'Not found'});
    }
  });
});

router.post('/material/new', (req, res, next) => {
  if (!req.auth(res, ['write', 'maintain', 'dev', 'admin'], 'basic')) return;

  // validate input
  const {error, value: material} = MaterialValidate.input(req.body, 'new');
  if(error !== undefined) {
    res.status(400).json({status: 'Invalid body format'});
    return;
  }

  MaterialModel.find({name: material.name}).lean().exec((err, data) => {
    if(err) next(err);
    if (data.length > 0) {
      res.status(400).json({status: 'Material name already taken'});
      return;
    }

    new MaterialModel(material).save((err, data) => {
      if(err) next(err);
      res.json(MaterialValidate.output(data.toObject()));
    });
  })
})


module.exports = router;